# This script generates the PyQt configuration and generates the Makefiles.
#
# Copyright (c) 2006
# 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
# 
# This file is part of PyQt.
# 
# This copy of PyQt is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2, or (at your option) any later
# version.
# 
# PyQt is supplied in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# 
# You should have received a copy of the GNU General Public License along with
# PyQt; see the file LICENSE.  If not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


import sys
import os
import string
import glob
import getopt
import shutil

import sipconfig


# Initialise the globals.
pyqt_version = 0x040000
pyqt_version_str = "snapshot-20060216"

sip_min_version = 0x040400


# Try and find a Qt installation to use as the default.  First look for Qt v4
# by looking for the first qmake on the path.  If this fails then check the
# QTDIR environment variable for an earlier version of Qt.
try:
    path = os.environ["PATH"]
except KeyError:
    path = ""

for d in path.split(os.pathsep):
    if os.access(os.path.join(d, "qmake"), os.X_OK):
        qt_dir = os.path.dirname(d)
        break
else:
    try:
        qt_dir = os.environ["QTDIR"]
    except KeyError:
        qt_dir = ""


qt_version = 0
qt_edition = ""
qt_incdir = None
qt_libdir = None
qt_winconfig = ""
qt_lib = None

pyqt = None
qt_sip_flags = []

# Get the SIP configuration.
sipcfg = sipconfig.Configuration()

# Command line options.
opt_qconfigdir = None
opt_pyqtbindir = sipcfg.default_bin_dir
opt_pyqtmoddir = sipcfg.default_mod_dir
opt_pyqtsipdir = sipcfg.default_sip_dir
opt_static = 0
opt_debug = 0
opt_concat = 0
opt_split = 1
opt_tracing = 0
opt_verbose = 0

opt_vendorcheck = 0
opt_vendincdir = sipcfg.py_inc_dir
opt_vendlibdir = sipcfg.py_lib_dir


def usage(rcode = 2):
    """Display a usage message and exit.

    rcode is the return code passed back to the calling process.
    """
    if qt_dir:
        def_qt_dir = qt_dir
    else:
        def_qt_dir = "none"

    print "Usage:"
    print "    python configure.py [-h] [-b dir] [-c] [-d dir] [-g dir] [-i] [-j #] [-k] [-l dir] [-m dir] [-q dir] [-r] [-u] [-v dir] [-w] option=value option+=value ..."
    print "where:"
    print "  -h      display this help message"
    print "  -b dir  where pyuic4, pyrcc4 and pylupdate4 will be installed [default %s]" % opt_pyqtbindir
    print "  -c      concatenate each module's C/C++ source files"
    print "  -d dir  where the PyQt modules will be installed [default %s]" % opt_pyqtmoddir
    print "  -g dir  where the Qt qconfig.h file can be found [default Qt include directory]"
    print "  -i      enable checking of signed interpreters using the VendorID package [default disabled]"
    print "  -j #    split the concatenated C++ source files into # pieces [default 1]"
    print "  -k      build the PyQt modules as static libraries"
    print "  -l dir  the directory containing the VendorID header file [default %s]" % opt_vendincdir
    print "  -m dir  the directory containing the VendorID library [default %s]" % opt_vendlibdir
    print "  -q dir  the root directory of the Qt installation [default %s]" % def_qt_dir
    print "  -r      generate code with tracing enabled [default disabled]"
    print "  -u      build with debugging symbols"
    print "  -v dir  where the PyQt .sip files will be installed [default %s]" % opt_pyqtsipdir
    print "  -w      don't suppress compiler output during configuration"

    sys.exit(rcode)


class ConfigureBase:
    """This is the base class for all PyQt version specific configurer classes.
    Anything here is common to all configurers.
    """
    def check_modules(self):
        """Check which modules should be built and add them to the global list.
        """
        pass

    def qt_version_tags(self):
        """Get the versions tags for the configuration.

        Returns a dictionary of versions and corresponding tags.
        """
        return {}

    def code(self):
        """Generate the code for a configuration.
        """
        pass

    def tools(self):
        """Create the Makefiles for any other sub-directories and return a list
        of those directories.

        Returns a list of sub-directories with Makefile.
        """
        return []

    def module_dir(self):
        """Return the configuration's module directory.
        """
        return opt_pyqtmoddir

    def module_installs(self):
        """Return a list of files to install in the module directory other than
        the modules themselves.
        """
        return ["pyqtconfig.py"]

    def sip_dir(self):
        """Return the configuration's .sip files directory.
        """
        return opt_pyqtsipdir


class ConfigurePyQt4(ConfigureBase):
    """This class defines the methods to configure PyQt v4.
    """
    def check_modules(self):
        sipcfg.pyqt_modules.append("QtCore")

        check_module("QtGui", "qwidget.h", "QWidget()")
        check_module("QtAssistant", "qassistantclient.h", "QAssistantClient(\"foo\")")
        check_module("QtNetwork", "qhostaddress.h", "QHostAddress()")
        check_module("QtOpenGL", "qgl.h", "QGLWidget()")
        check_module("QtSql", "qsqldatabase.h", "QSqlDatabase()")
        check_module("QtSvg", "qsvgwidget.h", "QSvgWidget()")
        check_module("QtXml", "qdom.h", "QDomDocument()")

    def code(self):
        if opt_vendorcheck:
            generate_code("QtCore", opt_vendincdir, opt_vendlibdir, "vendorid")
        else:
            generate_code("QtCore")

        if "QtGui" in sipcfg.pyqt_modules:
            generate_code("QtGui")

        if "QtAssistant" in sipcfg.pyqt_modules:
            generate_code("QtAssistant")

        if "QtNetwork" in sipcfg.pyqt_modules:
            generate_code("QtNetwork")

        if "QtOpenGL" in sipcfg.pyqt_modules:
            generate_code("QtOpenGL")

        if "QtSvg" in sipcfg.pyqt_modules:
            generate_code("QtSvg")

        if "QtSql" in sipcfg.pyqt_modules:
            generate_code("QtSql")

        if "QtXml" in sipcfg.pyqt_modules:
            generate_code("QtXml")

    def module_dir(self):
        return os.path.join(ConfigureBase.module_dir(self), "PyQt4")

    def module_installs(self):
        return ConfigureBase.module_installs(self) + ["__init__.py"]

    def sip_dir(self):
        return os.path.join(ConfigureBase.sip_dir(self), "PyQt4")

    def tools(self):
        sipconfig.inform("Creating elementtree Makefile...")

        makefile = sipconfig.PythonModuleMakefile(
            configuration=sipcfg,
            dstdir=os.path.join(self.module_dir(), "elementtree"),
            dir="elementtree"
        )

        makefile.generate()

        sipconfig.inform("Creating pyuic4 wrapper...")

        uicdir=os.path.join(self.module_dir(), "uic")
        wrapper = sipconfig.create_wrapper(os.path.join(uicdir, "pyuic.py"), os.path.join("pyuic", "pyuic4"), 1)

        sipconfig.inform("Creating pyuic4 Makefile...")

        makefile = sipconfig.PythonModuleMakefile(
            configuration=sipcfg,
            dstdir=uicdir,
            srcdir="uic",
            dir="pyuic",
            installs=[[os.path.basename(wrapper), opt_pyqtbindir]]
        )

        makefile.generate()

        sipconfig.inform("Creating pylupdate4 Makefile...")

        makefile = sipconfig.ProgramMakefile(
            configuration=sipcfg,
            build_file="pylupdate.sbf",
            dir="pylupdate",
            install_dir=opt_pyqtbindir,
            console=1,
            qt=["QtXml", "QtGui"],
            warnings=1
        )

        makefile.generate()

        sipconfig.inform("Creating pyrcc4 Makefile...")

        makefile = sipconfig.ProgramMakefile(
            configuration=sipcfg,
            build_file="pyrcc.sbf",
            dir="pyrcc",
            install_dir=opt_pyqtbindir,
            console=1,
            qt=["QtXml"],
            warnings=1
        )

        makefile.generate()

        return ["elementtree", "pyuic", "pylupdate", "pyrcc"]


def inform_user():
    """Tell the user the option values that are going to be used.
    """
    if qt_edition:
        edstr = qt_edition + " edition "
    else:
        edstr = ""

    sipconfig.inform("Qt v%s %sis being used." % (sipconfig.version_to_string(qt_version), edstr))
    sipconfig.inform("SIP %s is being used." % sipcfg.sip_version_str)
    sipconfig.inform("The Qt header files are in %s." % qt_incdir)
    sipconfig.inform("The %s Qt library is in %s." % (qt_lib, qt_libdir))
    sipconfig.inform("These PyQt modules will be built: %s." % string.join(sipcfg.pyqt_modules))

    sipconfig.inform("The PyQt modules will be installed in %s." % opt_pyqtmoddir)
    sipconfig.inform("The PyQt .sip files will be installed in %s." % opt_pyqtsipdir)

    sipconfig.inform("pyuic4, pyrcc4 and pylupdate4 will be installed in %s." % opt_pyqtbindir)

    if opt_vendorcheck:
        sipconfig.inform("PyQt will only be usable with signed interpreters.")


def create_config(module, template, macros):
    """Create the PyQt configuration module so that it can be imported by build
    scripts.

    module is the module file name.
    template is the template file name.
    macros is the dictionary of platform specific build macros.
    """
    sipconfig.inform("Creating %s..." % module)

    content = {
        "pyqt_version":       pyqt_version,
        "pyqt_version_str":   pyqt_version_str,
        "pyqt_bin_dir":       opt_pyqtbindir,
        "pyqt_mod_dir":       opt_pyqtmoddir,
        "pyqt_sip_dir":       opt_pyqtsipdir,
        "pyqt_modules":       sipcfg.pyqt_modules,
        "pyqt_qt_sip_flags":  qt_sip_flags,
        "qt_version":         qt_version,
        "qt_edition":         qt_edition,
        "qt_winconfig":       qt_winconfig,
        "qt_threaded":        1,
        "qt_dir":             qt_dir,
        "qt_inc_dir":         qt_incdir,
        "qt_lib_dir":         qt_libdir
    }

    if "qtaxcontainer" in sipcfg.pyqt_modules:
        content["pyqt_qtaxcontainer_sip_flags"] = qt_sip_flags

    if "qtcanvas" in sipcfg.pyqt_modules:
        content["pyqt_qtcanvas_sip_flags"] = qt_sip_flags

    if "qtgl" in sipcfg.pyqt_modules:
        content["pyqt_qtgl_sip_flags"] = qt_sip_flags

    if "qtnetwork" in sipcfg.pyqt_modules:
        content["pyqt_qtnetwork_sip_flags"] = qt_sip_flags

    if "qtsql" in sipcfg.pyqt_modules:
        content["pyqt_qtsql_sip_flags"] = qt_sip_flags

    if "qttable" in sipcfg.pyqt_modules:
        content["pyqt_qttable_sip_flags"] = qt_sip_flags

    if "qtui" in sipcfg.pyqt_modules:
        content["pyqt_qtui_sip_flags"] = qt_sip_flags

    if "qtxml" in sipcfg.pyqt_modules:
        content["pyqt_qtxml_sip_flags"] = qt_sip_flags

    sipconfig.create_config_module(module, template, content, macros)


def compile_qt_program(name, mname):
    """Compile a simple Qt application.

    name is the name of the single source file.
    mname is the name of the Qt module.

    Returns the name of the executable suitable for running or None if it
    wasn't created.
    """
    opengl = (mname == "QtOpenGL")

    makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=[mname], warnings=0, opengl=opengl, debug=opt_debug)

    exe, build = makefile.build_command(name)

    # Make sure the executable file doesn't exist.
    try:
        os.remove(exe)
    except OSError:
        pass

    if not opt_verbose and hasattr(os, "popen4"):
        fin, fout = os.popen4(build)

        # Read stdout and stderr until there is no more output.
        lout = fout.readline()
        while lout:
            lout = fout.readline()

        fin.close()
        fout.close()

        try:
            os.wait()
        except AttributeError:
            pass
    else:
        os.system(build)

    if not os.access(exe, os.X_OK):
        return None

    if sys.platform != "win32":
        exe = "./" + exe

    return exe


def check_vendorid():
    """See if the VendorID library and include file can be found.
    """
    global opt_vendorcheck

    if opt_vendorcheck:
        if os.access(os.path.join(opt_vendincdir, "vendorid.h"), os.F_OK):
            if glob.glob(os.path.join(opt_vendlibdir, "*vendorid*")):
                sipconfig.inform("The VendorID package was found.")
            else:
                opt_vendorcheck = 0
                sipconfig.inform("The VendorID library could not be found in %s and so signed interpreter checking will be disabled. If the VendorID library is installed then use the -m argument to explicitly specify the correct directory." % opt_vendlibdir)
        else:
            opt_vendorcheck = 0
            sipconfig.inform("vendorid.h could not be found in %s and so signed interpreter checking will be disabled. If the VendorID package is installed then use the -l argument to explicitly specify the correct directory." % opt_vendincdir)


def check_module(mname, incfile, ctor):
    """See if a module can be built and, if so, add it to the global list of
    modules.

    mname is the name of the module.
    incfile is the name of the include file needed for the test.
    ctor is the C++ constructor of the class being used for the test.
    """
    # Check the module's main .sip file exists.
    if os.access(os.path.join("sip", mname, mname + "mod.sip"), os.F_OK):
        sipconfig.inform("Checking to see if the %s module should be built..." % mname)

        if check_class(incfile, ctor, mname):
            sipcfg.pyqt_modules.append(mname)


def check_class(incfile, ctor, mname):
    """Return non-zero if a class is available.

    incfile is the name of the include file needed for the test.
    ctor is the C++ constructor of the class.
    mname is the name of the module.
    """
    cfgtest = "cfgtest.cpp"

    f = open(cfgtest, "w")

    f.write("""#include <%s>

int main(int argc, char **argv)
{
    new %s;
}
""" % (incfile, ctor))

    f.close()

    return compile_qt_program(cfgtest, mname)


def set_sip_flags():
    """Set the SIP platform, version and feature flags.
    """
    # If we don't check for signed interpreters, we exclude the 'VendorID'
    # feature
    if not opt_vendorcheck:
        qt_sip_flags.append("-x")
        qt_sip_flags.append("VendorID")

    # Handle the platform tag.
    if sys.platform == "win32":
        plattag = "WS_WIN"
    elif sys.platform == "darwin" and sipcfg.platform != "darwin-g++":
        plattag = "WS_MACX"
    else:
        plattag = "WS_X11"

    qt_sip_flags.append("-t")
    qt_sip_flags.append(plattag)

    # Handle the Qt version tag.
    qttags = pyqt.qt_version_tags()

    if qttags:
        qt_sip_flags.append("-t")
        qt_sip_flags.append(sipconfig.version_to_sip_tag(qt_version, qttags, "Qt"))


def generate_code(mname, extra_include_dir=None, extra_lib_dir=None, extra_lib=None):
    """Generate the code for a module.

    mname is the name of the module.
    extra_include_dir is an optional directory to add to the list of include
    directories.
    extra_lib_dir is an optional directory to add to the list of library
    directories.
    extra_lib is an optional library to add to the list of libraries.
    """
    sipconfig.inform("Generating the C++ source for the %s module..." % mname)

    try:
        shutil.rmtree(mname)
    except:
        pass

    try:
        os.mkdir(mname)
    except:
        sipconfig.error("Unable to create the %s directory." % mname)

    # Build the SIP command line.
    argv = ['"' + sipcfg.sip_bin + '"']

    argv.extend(qt_sip_flags)

    if opt_concat:
        argv.append("-j")
        argv.append(str(opt_split))

    if opt_tracing:
        argv.append("-r")

    argv.append("-c")
    argv.append(mname)

    buildfile = os.path.join(mname, mname + ".sbf")
    argv.append("-b")
    argv.append(buildfile)

    argv.append("-I")
    argv.append("sip")

    # SIP assumes POSIX style path separators.
    argv.append(string.join(["sip", mname, mname + "mod.sip"], "/"))

    os.system(string.join(argv))

    # Check the result.
    if not os.access(buildfile, os.F_OK):
        sipconfig.error("Unable to create the C++ code.")

    # Generate the Makefile.
    sipconfig.inform("Creating the Makefile for the %s module..." % mname)

    installs = []
    sipfiles = []

    for s in glob.glob("sip/" + mname + "/*.sip"):
        sipfiles.append(os.path.join("..", "sip", mname, os.path.basename(s)))

    installs.append([sipfiles, os.path.join(pyqt.sip_dir(), mname)])

    opengl = (mname == "QtOpenGL")

    makefile = sipconfig.SIPModuleMakefile(
        configuration=sipcfg,
        build_file=mname + ".sbf",
        dir=mname,
        install_dir=pyqt.module_dir(),
        installs=installs,
        qt=[mname],
        opengl=opengl,
        warnings=1,
        static=opt_static,
        debug=opt_debug
    )

    if extra_include_dir:
        makefile.extra_include_dirs.append(extra_include_dir)

    if extra_lib_dir:
        makefile.extra_lib_dirs.append(extra_lib_dir)

    if extra_lib:
        makefile.extra_libs.append(extra_lib)

    makefile.generate()


def check_license():
    """Handle the validation of the PyQt license.
    """
    try:
        import license
        ltype = license.LicenseType
        lname = license.LicenseName

        try:
            lfile = license.LicenseFile
        except AttributeError:
            lfile = None
    except ImportError:
        ltype = None

    if ltype is None:
        ltype = "GPL"
        lname = "GNU General Public License"
        lfile = None

    sipconfig.inform("This is the %s version of PyQt %s (licensed under the %s) for Python %s on %s." % (ltype, pyqt_version_str, lname, string.split(sys.version)[0], sys.platform))

    # Common checks.
    try:
        qted = qt_edition
    except AttributeError:
        qted = None

    if qted and ltype != "internal":
        if (qted == "free" and ltype != "GPL") or (qted != "free" and ltype == "GPL"):
            sipconfig.error("This version of PyQt and the %s edition of Qt have incompatible licenses." % qted)

    # Confirm the license.
    print
    print "Type 'L' to view the license."
    print "Type 'yes' to accept the terms of the license."
    print "Type 'no' to decline the terms of the license."
    print

    while 1:
        try:
            resp = raw_input("Do you accept the terms of the license? ")
        except:
            resp = ""

        resp = string.lower(string.strip(resp))

        if resp == "yes":
            break

        if resp == "no":
            sys.exit(0)

        if resp == "l":
            os.system("more LICENSE")

    # If there should be a license file then check it is where it should be.
    if lfile:
        if os.access(os.path.join("sip", lfile), os.F_OK):
            sipconfig.inform("Found the license file %s." % lfile)
        else:
            sipconfig.error("Please copy the license file %s to the sip directory." % lfile)


def get_build_macros(overrides):
    """Return the dictionary of platform specific build macros from the Qt
    installation.  Return None if any of the overrides was invalid.

    overrides is a list of macros overrides from the user.
    """
    # Get the name of the qmake configuration file to take the macros from.
    if "QMAKESPEC" in os.environ.keys():
        fname = os.path.join(qt_dir, "mkspecs", os.environ["QMAKESPEC"], "qmake.conf")
    else:
        fname = os.path.join(qt_dir, "mkspecs", "default", "qmake.conf")

        if not os.access(fname, os.F_OK):
            sipconfig.error("Unable to find the default configuration file %s. You can use the QMAKESPEC environment variable to specify the correct platform instead of \"default\"." % fname)

    # Add the Qt specific macros to the default.
    names = sipcfg.build_macros().keys()
    names.append("INCDIR_QT")
    names.append("LIBDIR_QT")
    names.append("MOC")

    # Make sure $QTDIR reflects any directory passed on the command line.
    os.environ["QTDIR"] = qt_dir

    properties = {
        "QT_INSTALL_BINS":      os.path.join(qt_dir, "bin"),
        "QT_INSTALL_HEADERS":   os.path.join(qt_dir, "include"),
        "QT_INSTALL_LIBS":      os.path.join(qt_dir, "lib")
    }

    return sipconfig.parse_build_macros(fname, names, overrides, properties)


def qt_include_file(fname, fallback):
    """In some Qt4 installations the public header files just have a single
    line that #includes the real file in the src directory.  Rather than parse
    #include directives we just look at the size of the file and assume it
    contains a #include if it seems too small to contain anything useful.

    fname is the file to read.
    fallback is the file to fall back to if fname is too small.

    Returns the file to use or None if it wasn't found.
    """
    try:
        sbuf = os.stat(fname)
    except OSError:
        sbuf = None

    if sbuf:
        if sbuf.st_size < 100 and os.access(fallback, os.F_OK):
            fname = fallback
    else:
        fname = None

    return fname


def check_qt_installation(macros):
    """Check the Qt installation and get the version number and edition.

    macros is the dictionary of build macros.
    """
    # Get the Makefile generator.
    generator = macros["MAKEFILE_GENERATOR"]

    # Set the Qt include and lib directories.
    global qt_incdir, qt_libdir

    qt_incdir = macros["INCDIR_QT"]

    if not qt_incdir:
        qt_incdir = os.path.join(qt_dir, "include")
        macros["INCDIR_QT"] = qt_incdir

    qt_libdir = macros["LIBDIR_QT"]

    if not qt_libdir:
        qt_libdir = os.path.join(qt_dir, "lib")
        macros["LIBDIR_QT"] = qt_libdir

    # Check the Qt header files have been installed.  We look for Qt3 as well
    # so that we can give the user a better error message if they are trying to
    # use it.
    qt4_d = os.path.join(qt_incdir, "QtCore")

    qglobal = qt_include_file(os.path.join(qt4_d, "qglobal.h"),
                              os.path.join(qt_dir, "src", "corelib", "global", "qglobal.h"))

    if not qglobal:
        qglobal = os.path.join(qt_incdir, "qglobal.h")

        if not os.access(qglobal, os.F_OK):
            sipconfig.error("qglobal.h could not be found in %s or %s." % (qt4_d, qt_incdir))

    # Get the Qt version number.
    global qt_version

    qt_version, ignore = sipconfig.read_version(qglobal, "Qt", "QT_VERSION")

    if qt_version < 0x040000:
        sipconfig.error("Qt v3.x and earlier require PyQt v3.x.")

    if qt_version < 0x040100:
        sipconfig.error("PyQt v4 requires Qt v4.1.0 or later.")

    # Try and work out which edition it is.
    global qt_edition

    # See if it is the open source edition.
    if opt_qconfigdir:
        qconfigdir = opt_qconfigdir
    else:
        qconfigdir = os.path.join(qt_incdir, "QtCore")

    qconfig = qt_include_file(os.path.join(qconfigdir, "qconfig.h"),
                              os.path.join(qt_dir, "src", "corelib", "global", "qconfig.h"))

    if not os.access(qconfig,os.F_OK):
        sipconfig.error("qconfig.h could not be found in %s." % qconfigdir)

    f = open(qconfig)
    l = f.readline()

    while l:
        wl = string.split(l)

        if len(wl) == 3 and wl[0] == "#define":
            name_val = wl[1:]
        elif len(wl) == 4 and wl[0] == "#" and wl[1] == "define":
            name_val = wl[2:]
        else:
            name_val = None

        if name_val and name_val[0] == "QT_EDITION" and name_val[1] == "QT_EDITION_OPENSOURCE":
            qt_edition = "free"
            break

        l = f.readline()

    f.close()

    if not qt_edition:
        # Work out which commercial edition it is.
        if not os.access(os.path.join(qt_incdir, "QtGui"), os.F_OK):
            qt_edition = "Console"
        elif not os.access(os.path.join(qt_incdir, "QtXml"), os.F_OK):
            qt_edition = "Desktop Light"
        else:
            qt_edition = "Desktop"

    # Determine the Qt library to link against.
    resolve_qt4_library(generator)

    # Try and work out how Qt was built on Windows.
    if sys.platform == "win32":
        global qt_winconfig

        try:
            f = open(os.path.join(qt_dir, ".qtwinconfig"), "r")
        except IOError:
            f = None

        if f:
            cfg = f.readline()
            f.close()

            val = string.find(cfg, "=")

            if val >= 0:
                qt_winconfig = string.strip(cfg[val + 1:])
        else:
            # Assume it was built as a DLL.
            qt_winconfig = "shared"

    global pyqt

    pyqt = ConfigurePyQt4()

    # We haven't yet factored out sipconfig's knowledge of how to build Qt
    # binaries and it is expecting to find these in the configuration when it
    # generates the Makefiles.
    sipcfg.qt_version = qt_version
    sipcfg.qt_edition = qt_edition
    sipcfg.qt_winconfig = qt_winconfig
    sipcfg.qt_threaded = 1
    sipcfg.qt_dir = qt_dir
    sipcfg.qt_lib_dir = qt_libdir
    sipcfg.pyqt_modules = []


def resolve_qt4_library(generator):
    """Check that the Qt v4 library can be found.

    generator is the name of the Makefile generator.
    """
    global qt_lib

    qt_lib = "QtCore"

    if opt_debug and sys.platform != "win32":
        qt_lib = qt_lib + "_debug"

    if not is_qt_library(generator, qt_lib):
        sipconfig.error("The %s Qt library could not be found in %s." % (qt_lib, qt_libdir))


def is_qt_library(generator, lib):
    """See if a particular Qt library is installed.

    generator is the name of the Makefile generator.
    lib is the name of the library.
    """
    if generator in ("MSVC", "MSVC.NET", "BMAKE"):
        lpatts = [lib + "[0-9]*.lib", lib + ".lib"]
    elif generator == "MINGW":
        lpatts = ["lib" + lib + "[0-9]*.a"]
    else:
        lpatts = ["lib" + lib + ".*"]

    for lpatt in lpatts:
        lmatch = glob.glob(os.path.join(qt_libdir, lpatt))

        if lmatch:
            return lmatch

    return []


def main(argv):
    """Create the configuration module module.

    argv is the list of command line arguments.
    """
    # Check SIP is new enough.
    if sipcfg.sip_version_str[:8] != "snapshot":
        if sipcfg.sip_version < sip_min_version:
            sipconfig.error("This version of PyQt requires SIP v%s or later" % sipconfig.version_to_string(sip_min_version))

    # Parse the command line.
    try:
        optlist, args = getopt.getopt(argv[1:], "hb:cd:g:ij:kl:m:q:ruv:w")
    except getopt.GetoptError:
        usage()

    global qt_dir, opt_qconfigdir, opt_pyqtbindir, opt_pyqtmoddir
    global opt_pyqtsipdir, opt_static, opt_debug, opt_concat, opt_split
    global opt_tracing, opt_verbose, opt_vendorcheck, opt_vendincdir
    global opt_vendlibdir

    for opt, arg in optlist:
        if opt == "-h":
            usage(0)
        elif opt == "-b":
            opt_pyqtbindir = os.path.abspath(arg)
        elif opt == "-c":
            opt_concat = 1
        elif opt == "-d":
            opt_pyqtmoddir = os.path.abspath(arg)
        elif opt == "-g":
            opt_qconfigdir = os.path.abspath(arg)
        elif opt == "-i":
            opt_vendorcheck = 1
        elif opt == "-j":
            try:
                opt_split = int(arg)
            except:
                usage()
        elif opt == "-k":
            opt_static = 1
        elif opt == "-l":
            opt_vendincdir = arg
        elif opt == "-m":
            opt_vendlibdir = arg
        elif opt == "-q":
            qt_dir = os.path.abspath(arg)
        elif opt == "-r":
            opt_tracing = 1
        elif opt == "-u":
            opt_debug = 1
        elif opt == "-v":
            opt_pyqtsipdir = os.path.abspath(arg)
        elif opt == "-w":
            opt_verbose = 1

    # Check that we know the name of the Qt root directory.
    if not qt_dir:
        sipconfig.error("A Qt installation could not be found. Use use the -q argument to explicitly specify the correct directory.")

    # When building static libraries, signed interpreter checking makes no
    # sense.
    if opt_vendorcheck and opt_static:
        sipconfig.error("Using the VendorID package when building static libraries makes no sense.")

    # Replace the existing build macros with the ones from the Qt installation.
    macros = get_build_macros(args)

    if macros is None:
        usage()

    sipcfg.set_build_macros(macros)

    # Check Qt is what we need.
    check_qt_installation(macros)

    # Check the licenses are compatible.
    check_license()

    # Check which modules to build.
    pyqt.check_modules()

    # Check for the VendorID package.
    check_vendorid()

    # Set the SIP platform, version and feature flags.
    set_sip_flags()

    # Tell the user what's been found.
    inform_user()

    # Generate the code.
    pyqt.code()

    # Create the additional Makefiles.
    sipconfig.inform("Creating top level Makefile...")

    sipconfig.ParentMakefile(
        configuration=sipcfg,
        subdirs=sipcfg.pyqt_modules + pyqt.tools(),
        installs=(pyqt.module_installs(), pyqt.module_dir())
    ).generate()

    # Install the configuration module.
    create_config("pyqtconfig.py", "pyqtconfig.py.in", macros)


###############################################################################
# The script starts here.
###############################################################################

if __name__ == "__main__":
    try:
        main(sys.argv)
    except SystemExit:
        raise
    except:
        print \
"""An internal error occured.  Please report all the output from the program,
including the following traceback, to support@riverbankcomputing.co.uk.
"""
        raise
