# Hic sunt leones!
import re
import sys

from indenter import write_code

# i18n_strings is used by UICompiler.finalize
i18n_strings = []
# translator_function is set by UICompiler
translator_function = "tr"
def i18n_print(string):
    i18n_strings.append(string)
    
def moduleMember(module, name):
    if module == "":
        return name
    else:
        return "%s.%s" % (module, name)

def obj_to_argument(obj):
    if isinstance(obj, str):
        return "\"%s\"" % (obj,)
    else:
        return str(obj)
    
class Literal(object):
    """Literal(string) -> new literal

    string will not be quoted when put into an argument list"""
    def __init__(self, string):
        self.string = string

    def __str__(self):
        return self.string

class i18n_string(object):
    """i18n_string(string)

    string will be escaped, quoted and translated when included into
    a function call argument list."""
    _esc_regex = re.compile(r"(\"|\')")
    def __init__(self, string):
        self.string = string

    def escape(self, text):
        x = self._esc_regex.sub(r"\\\1", text)
        return re.sub(r"\n", r"\\n", x)
        
    def __str__(self):
        return "%s(\"%s\")" % (translator_function, self.escape(self.string))


# classes with this flag will occur in retranslateUi completely
# (construction, function calls, all calls to other functions with this
#  class as an argument)
I18N_ONLY = 1
# classes with this flag will be handled as literal values. If functions are
# called on these classes, the literal value changes.
# Example:
# the code
# >>> QSize(9,10).expandedTo(...)
# will print just that code.
AS_ARGUMENT = 2

# ATTENTION: currently, classes can either be literal or normal. If a class
# should need both kinds of behaviour, the code has to be changed.

class ProxyClassMember(object):
    def __init__(self, proxy, function_name, flags):
        self.proxy = proxy
        self.function_name = function_name
        self.flags = flags
        
    def __str__(self):
        return "%s.%s" % (self.proxy, self.function_name)
    
    def __call__(self, *args):
        func_call = "%s.%s(%s)" % (self.proxy,
                                   self.function_name,
                                   ",".join(map(obj_to_argument, args)))
        if self.flags & AS_ARGUMENT:
            self.proxy._name = func_call
            return self.proxy
        else:
            needs_translation = self.flags & I18N_ONLY
            for arg in args:
                if isinstance(arg, i18n_string) or getattr(arg, "flags", 0) & I18N_ONLY:
                    needs_translation = True
            if needs_translation:
                i18n_print(func_call)
            else:
                write_code(func_call)


class ProxyType(type):
    def __getattribute__(cls, attribute):
        try:
            return type.__getattribute__(cls, attribute)
        except AttributeError:
            return ProxyClassMember(moduleMember(cls.module, cls.__name__),
                                    attribute, 0)


class ProxyClass(object):
    __metaclass__ = ProxyType
    flags = 0
    module = "QtCore"
    def __init__(self, objectname, is_attribute,
                 args = (), noInstantiation = False):
        if is_attribute:
            self._name = "self.%s" % (objectname, )
        else:
            self._name = objectname
        if not noInstantiation:
            funcall = "\n%s = %s(%s)" % \
                      (self._name,
                       moduleMember(self.module, self.__class__.__name__),
                       ",".join(map(str, args)))
            if self.flags & I18N_ONLY:
                i18n_print(funcall)
            else:
                write_code(funcall)
    
    def __str__(self):
        return self._name

    def __getattribute__(self, attribute):
        try:
            return object.__getattribute__(self, attribute)
        except AttributeError:
            return ProxyClassMember(self, attribute, self.flags)

class LiteralProxyClass(ProxyClass):
    """LiteralObject(*args) -> new literal class

    a literal class can be used as argument in a function call

    >>> class Foo(LiteralProxyClass): pass
    >>> str(Foo(1,2,3)) == "Foo(1,2,3)"
    """
    flags = AS_ARGUMENT
    def __init__(self, *args):
        self._name = "%s(%s)" % \
                     (moduleMember(self.module, self.__class__.__name__),
                      ",".join(map(obj_to_argument, args)))
            
# These are all the Qt classes used by pyuic4, both from QtCore and QtGui.
# If a class is missing, the compiler will fail, normally with an
# AttributeError.
# For adding new classes:
#     - utility classes used as literal values inherit from LiteralProxyClass
#     - classes which are *not* QWidgets inherit from ProxyClass
#     - new QWidget-derived classes have to inherit from qtproxies.QWidget
#       If the widget does not need any special methods, it can be listed
#       in _qwidgets
class QVariant(LiteralProxyClass): module = "QtCore"
class SIGNAL(LiteralProxyClass): module = "QtCore"
class SLOT(LiteralProxyClass): module = "QtCore"
class QRect(LiteralProxyClass): module = "QtCore"
class QSize(LiteralProxyClass): module = "QtCore"
class QColor(LiteralProxyClass): module = "QtGui"
class QCursor(LiteralProxyClass): module = "QtGui"
class QIcon(LiteralProxyClass): module = "QtGui"
class QPixmap(LiteralProxyClass): module = "QtGui"


class Qt(ProxyClass):
    class CursorShape(LiteralProxyClass):
        module = "QtCore.Qt"
    
class QListWidgetItem(ProxyClass):
    module = "QtGui"
    flags = I18N_ONLY

class QTreeWidgetItem(ProxyClass):
    module = "QtGui"
    flags = I18N_ONLY

class QTableWidgetItem(ProxyClass):
    module = "QtGui"
    flags = I18N_ONLY

class QPalette(ProxyClass):
    module = "QtGui"
    class ColorRole(LiteralProxyClass):
        module = "QtGui.QPalette"
    
class QFont(ProxyClass): module = "QtGui"
class QSpacerItem(ProxyClass): module = "QtGui"
class QSizePolicy(ProxyClass):
    module = "QtGui"
    class Policy(LiteralProxyClass):
        module = "QtGui.QSizePolicy"

class QMetaObject(ProxyClass):
    pass

class QObject(ProxyClass):
    def metaObject(self):
        class _FakeMetaObject(object):
            def className(*args):
                return self.__class__.__name__
        return _FakeMetaObject()

    def objectName(self):
        return self._name.split(".")[-1]

class QAction(QObject): module = "QtGui"
class QLayout(QObject): module = "QtGui"
class QGridLayout(QLayout): pass
class QHBoxLayout(QLayout): pass
class QVBoxLayout(QLayout): pass

class QWidget(QObject):
    module = "QtGui"
    def font(self):
        return Literal("%s.font()" % (self,))
    
    def minimumSizeHint(self):
        return Literal("%s.minimumSizeHint()" % (self,))
    
    def sizePolicy(self):
        sp = LiteralProxyClass()
        sp._name = "%s.sizePolicy()" % (self,)
        return sp

class QListWidget(QWidget):
    def clear(self):
        i18n_print("%s.clear()" % (self,))
        
class QTreeWidget(QWidget):
    def clear(self):
        i18n_print("%s.clear()" % (self,))
    
    def headerItem(self):
        return QWidget("%s.headerItem()" % (self,), False, (), noInstantiation = True)

class QMenu(QWidget):
    def menuAction(self):
        return Literal("%s.menuAction()" % (self,))

class QTabWidget(QWidget):
    def addTab(self, *args):
        write_code("%s.addTab(%s, \"\")" % (self._name, args[0]))
        i18n_print("%s.setTabText(%s.indexOf(%s), %s)" % \
                   (self._name, self._name, args[0], args[1]))

class QToolBox(QWidget):
    def addItem(self, *args):
        write_code("%s.addItem(%s, \"\")" % (self._name, args[0]))
        i18n_print("%s.setItemText(%s.indexOf(%s), %s)" % \
                   (self._name, self._name, args[0], args[1]))


# This list contains all QWidget-derived classes that can be created using
# Designer. If a widget does not already have its own class in this module,
# an empty class, derived from QWidget, is created in the module namespace.
_qwidgets = (
    "QCheckBox", "QComboBox", "QDateEdit", "QDateTimeEdit", "QDial", "QDialog",
    "QDockWidget", "QDoubleSpinBox", "QFrame", "QGroupBox", "QLabel",
    "QLCDNumber", "QLineEdit", "QListView", "QListWidget", "QMainWindow",
    "QMenu", "QMenuBar", "QProgressBar", "QPushButton", "QRadioButton", 
    "QScrollBar", "QSlider", "QSpinBox", "QSplitter", "QStackedWidget", 
    "QStatusBar", "QTableView", "QTableWidget", "QTabWidget", "QTextBrowser",
    "QTextEdit", "QTimeEdit", "QToolBar", "QToolBox", "QToolButton", "QTreeView",
    "QTreeWidget", "QAbstractItemView",)

for _class in _qwidgets:
    if not locals().has_key(_class):
        locals()[_class] = type(_class, (QWidget, ), {})
